@@ -389,12 +389,6 @@ kernel_interface_channel(const char *ifname, int ifindex)
     return -1;
 }
 
-int
-kernel_disambiguate(int v4)
-{
-    return 0;
-}
-
 int
 kernel_has_ipv6_subtrees(void)
 {
@@ -852,29 +846,6 @@ kernel_callback(struct kernel_filter *filter)
 
 }
 
-int
-add_rule(int prio, const unsigned char *src_prefix, int src_plen, int table)
-{
-    errno = ENOSYS;
-    return -1;
-}
-
-int
-flush_rule(int prio, int family)
-{
-    errno = ENOSYS;
-    return -1;
-}
-
-int
-change_rule(int new_prio, int old_prio,
-            const unsigned char *src, int plen, int table)
-{
-    errno = ENOSYS;
-    return -1;
-}
-
-
 /* Local Variables:      */
 /* c-basic-offset: 4     */
 /* indent-tabs-mode: nil */
