@@ -718,7 +718,7 @@ parse_packet(const unsigned char *from, struct interface *ifp,
                 }
                 have_router_id = 1;
             }
-            if(!have_router_id && message[2] != 0) {
+            if(metric < INFINITY && !have_router_id && message[2] != 0) {
                 fprintf(stderr, "Received prefix with no router id.\n");
                 goto fail;
             }
@@ -728,9 +728,13 @@ parse_packet(const unsigned char *from, struct interface *ifp,
                    format_prefix(prefix, plen),
                    format_address(from), ifp->name);
             if(message[2] == 1) {
-                if(!have_v4_nh)
-                    goto fail;
-                nh = v4_nh;
+                if(have_v4_nh) {
+                    nh = v4_nh;
+                } else {
+                    if(metric < INFINITY)
+                        goto fail;
+                    nh = NULL;
+                }
             } else if(have_v6_nh) {
                 nh = v6_nh;
             } else {
@@ -773,7 +777,8 @@ parse_packet(const unsigned char *from, struct interface *ifp,
                     goto done;
             }
 
-            update_route(router_id, prefix, plen, src_prefix, src_plen, seqno,
+            update_route(have_router_id ? router_id : NULL,
+                         prefix, plen, src_prefix, src_plen, seqno,
                          metric, interval, neigh, nh,
                          channels, channels_len);
         } else if(type == MESSAGE_REQUEST) {
@@ -1250,7 +1255,8 @@ really_buffer_update(struct buffered *buf, struct interface *ifp,
     if(channels_size > 0) {
         accumulate_byte(buf, 2);
         accumulate_byte(buf, channels_len);
-        accumulate_bytes(buf, channels, channels_len);
+        if(channels_len > 0)
+            accumulate_bytes(buf, channels, channels_len);
     }
     end_message(buf, MESSAGE_UPDATE, len);
     if(flags & 0x80) {
@@ -1425,9 +1431,9 @@ flushupdates(struct interface *ifp)
                     continue;
 
                 if(route_ifp->channel == IF_CHANNEL_NONINTERFERING) {
-                    memcpy(channels, route->channels,
-                           MIN(route->channels_len, MAX_CHANNEL_HOPS));
                     chlen = MIN(route->channels_len, MAX_CHANNEL_HOPS);
+                    if(chlen > 0)
+                        memcpy(channels, route->channels, chlen);
                 } else {
                     if(route_ifp->channel == IF_CHANNEL_UNKNOWN)
                         channels[0] = IF_CHANNEL_INTERFERING;
@@ -1567,7 +1573,7 @@ send_update(struct interface *ifp, int urgent,
         struct route_stream *routes;
         send_self_update(ifp);
         debugf("Sending update to %s for any.\n", ifp->name);
-        routes = route_stream(ROUTE_INSTALLED);
+        routes = route_stream(1);
         if(routes) {
             while(1) {
                 int is_ss;
